@@ -11,10 +11,9 @@ use crate::parser::{parse, Event, EventData, EventScope, Position};
 use crate::utils::INTERNAL_MODE;
 use crate::Result;
 
-use anyhow::{bail, Context};
+use anyhow::bail;
 use std::cell::RefCell;
 use std::collections::HashMap;
-use std::result::Result as StdResult;
 use std::sync::Arc;
 
 pub fn eval(
@@ -29,10 +28,10 @@ pub fn eval(
 
 pub fn export(source: &str) -> Result<serde_json::Value> {
     let cmd = Command::new(source)?;
-    cmd.to_json().with_context(|| "Failed to export json")
+    Ok(cmd.to_json())
 }
 
-#[derive(Default)]
+#[derive(Debug, Default)]
 pub struct Command {
     pub(crate) name: Option<String>,
     pub(crate) fn_name: Option<String>,
@@ -87,30 +86,26 @@ impl Command {
         Ok(matcher.to_arg_values())
     }
 
-    pub fn to_json(&self) -> StdResult<serde_json::Value, serde_json::Error> {
-        let subcommands: StdResult<Vec<serde_json::Value>, _> =
+    pub fn to_json(&self) -> serde_json::Value {
+        let subcommands: Vec<serde_json::Value> =
             self.subcommands.iter().map(|v| v.to_json()).collect();
-        let flag_option_params: StdResult<Vec<serde_json::Value>, _> = self
+        let flag_option_params: Vec<serde_json::Value> = self
             .flag_option_params
             .iter()
-            .map(serde_json::to_value)
+            .map(|v| v.to_json())
             .collect();
-        let positional_params: StdResult<Vec<serde_json::Value>, _> = self
-            .positional_params
-            .iter()
-            .map(serde_json::to_value)
-            .collect();
-        let value = serde_json::json!({
+        let positional_params: Vec<serde_json::Value> =
+            self.positional_params.iter().map(|v| v.to_json()).collect();
+        serde_json::json!({
             "describe": self.describe,
             "name": self.name,
             "author": self.author,
             "version": self.version,
-            "options": flag_option_params?,
-            "positionals": positional_params?,
+            "options": flag_option_params,
+            "positionals": positional_params,
             "aliases": self.aliases,
-            "subcommands": subcommands?,
-        });
-        Ok(value)
+            "subcommands": subcommands,
+        })
     }
 
     pub(crate) fn new_from_events(events: &[Event]) -> Result<Self> {
@@ -155,8 +150,8 @@ impl Command {
                     if param.is_option() {
                         root_data.borrow_mut().add_param_fn(
                             position,
-                            &param.default_fn,
-                            &param.choices_fn,
+                            param.default_fn(),
+                            param.choice_fn(),
                         );
                     }
                     cmd.names_checker.check_flag_option(&param, position)?;
@@ -166,8 +161,8 @@ impl Command {
                     let cmd = Self::get_cmd(&mut root_cmd, param.tag_name(), position)?;
                     root_data.borrow_mut().add_param_fn(
                         position,
-                        &param.default_fn,
-                        &param.choices_fn,
+                        param.default_fn(),
+                        param.choice_fn(),
                     );
                     cmd.add_positional_param(param, position)?;
                 }
@@ -193,7 +188,7 @@ impl Command {
                             bail!("{}(line {}) invalid command name", name, position);
                         } else if parts_len == 1 {
                             let cmd = root_cmd.subcommands.last_mut().unwrap();
-                            cmd.name = Some(parts[0].to_string());
+                            cmd.name = Some(sanitize_cmd_name(&name));
                             cmd.fn_name = Some(name.to_string());
                             for name in &cmd.aliases {
                                 if let Some(pos) = root_data.borrow().cmd_fns.get(name) {
@@ -211,9 +206,11 @@ impl Command {
                         } else {
                             let mut cmd = root_cmd.subcommands.pop().unwrap();
                             let (child, parents) = parts.split_last().unwrap();
-                            cmd.name = Some(child.to_string());
+                            let parents: Vec<String> =
+                                parents.iter().map(|v| sanitize_cmd_name(v)).collect();
+                            cmd.name = Some(sanitize_cmd_name(child));
                             cmd.fn_name = Some(name.to_string());
-                            match retrive_cmd(&mut root_cmd, parents) {
+                            match retrive_cmd(&mut root_cmd, &parents) {
                                 Some(parent_cmd) => {
                                     parent_cmd
                                         .subcommand_fns
@@ -288,7 +285,7 @@ impl Command {
         let required_options: Vec<String> = self
             .flag_option_params
             .iter()
-            .filter(|v| v.required)
+            .filter(|v| v.required())
             .map(|v| v.render_name_values())
             .collect();
         if self.flag_option_params.len() != required_options.len() {
@@ -339,10 +336,10 @@ impl Command {
         }
         let mut list = vec![];
         let mut any_describe = false;
-        let mut double_dash = true;
+        let mut single_hyphen = false;
         for param in self.flag_option_params.iter() {
-            if param.dashes == "-" {
-                double_dash = false;
+            if param.single_hyphen {
+                single_hyphen = true;
             }
             let value = param.render_body();
             let describe = param.render_describe();
@@ -351,8 +348,8 @@ impl Command {
             }
             list.push((value, describe));
         }
-        self.add_help_flag(&mut list, double_dash, any_describe);
-        self.add_version_flag(&mut list, double_dash, any_describe);
+        self.add_help_flag(&mut list, single_hyphen, any_describe);
+        self.add_version_flag(&mut list, single_hyphen, any_describe);
         output.push("OPTIONS:".to_string());
         let value_size = list.iter().map(|v| v.0.len()).max().unwrap_or_default() + 2;
         for (value, describe) in list {
@@ -423,20 +420,16 @@ impl Command {
     }
 
     pub(crate) fn list_names(&self) -> Vec<String> {
-        let mut output = vec![self.name.clone().unwrap_or_default()];
+        let mut output: Vec<String> = self.name.clone().into_iter().collect();
         output.extend(self.aliases.to_vec());
         output
     }
 
     pub(crate) fn list_subcommand_names(&self) -> Vec<String> {
-        let mut output = vec![];
-        for subcmd in self.subcommands.iter() {
-            if let Some(name) = subcmd.name.clone() {
-                output.push(name);
-            }
-            output.extend(subcmd.aliases.to_vec());
-        }
-        output
+        self.subcommands
+            .iter()
+            .flat_map(|v| v.list_names())
+            .collect()
     }
 
     pub(crate) fn find_subcommand(&self, name: &str) -> Option<&Self> {
@@ -453,19 +446,30 @@ impl Command {
     pub(crate) fn find_flag_option(&self, name: &str) -> Option<&FlagOptionParam> {
         self.flag_option_params
             .iter()
-            .find(|v| v.name == name || v.is_match(name))
+            .find(|v| v.name() == name || v.is_match(name))
+    }
+
+    pub(crate) fn find_prefixed_option(&self, name: &str) -> Option<(&FlagOptionParam, String)> {
+        for param in self.flag_option_params.iter() {
+            if let Some(prefix) = param.prefixed() {
+                if name.starts_with(&prefix) {
+                    return Some((param, prefix));
+                }
+            }
+        }
+        None
     }
 
     pub(crate) fn match_version_short_name(&self) -> bool {
         match self.find_flag_option("-V") {
-            Some(param) => &param.name == "version",
+            Some(param) => param.name() == "version",
             None => true,
         }
     }
 
     pub(crate) fn match_help_short_name(&self) -> bool {
         match self.find_flag_option("-h") {
-            Some(param) => &param.name == "help",
+            Some(param) => param.name() == "help",
             None => true,
         }
     }
@@ -503,7 +507,7 @@ impl Command {
             && self
                 .positional_params
                 .first()
-                .map(|v| v.terminated)
+                .map(|v| v.terminated())
                 .unwrap_or_default()
     }
 
@@ -548,18 +552,18 @@ impl Command {
     fn add_help_flag(
         &self,
         list: &mut Vec<(String, String)>,
-        double_dash: bool,
+        single_hyphen: bool,
         any_describe: bool,
     ) {
         if self.find_flag_option("help").is_some() {
             return;
         }
-        let dashes = if double_dash { "--" } else { " -" };
+        let hyphens = if single_hyphen { " -" } else { "--" };
         list.push((
             if self.match_help_short_name() {
-                format!("-h, {}help", dashes)
+                format!("-h, {}help", hyphens)
             } else {
-                format!("    {}help", dashes)
+                format!("    {}help", hyphens)
             },
             if any_describe {
                 "Print help".into()
@@ -572,7 +576,7 @@ impl Command {
     fn add_version_flag(
         &self,
         list: &mut Vec<(String, String)>,
-        double_dash: bool,
+        single_hyphen: bool,
         any_describe: bool,
     ) {
         if self.version.is_none() {
@@ -581,12 +585,12 @@ impl Command {
         if self.find_flag_option("version").is_some() {
             return;
         }
-        let dashes = if double_dash { "--" } else { " -" };
+        let hyphens = if single_hyphen { " -" } else { "--" };
         list.push((
             if self.match_version_short_name() {
-                format!("-V, {}version", dashes)
+                format!("-V, {}version", hyphens)
             } else {
-                format!("    {}version", dashes)
+                format!("    {}version", hyphens)
             },
             if any_describe {
                 "Print version".into()
@@ -597,17 +601,21 @@ impl Command {
     }
 }
 
-fn retrive_cmd<'a>(cmd: &'a mut Command, cmd_paths: &[&str]) -> Option<&'a mut Command> {
+fn retrive_cmd<'a>(cmd: &'a mut Command, cmd_paths: &[String]) -> Option<&'a mut Command> {
     if cmd_paths.is_empty() {
         return Some(cmd);
     }
     let child = cmd
         .subcommands
         .iter_mut()
-        .find(|v| v.name.as_deref() == Some(cmd_paths[0]))?;
+        .find(|v| v.name.as_deref() == Some(cmd_paths[0].as_str()))?;
     retrive_cmd(child, &cmd_paths[1..])
 }
 
+fn sanitize_cmd_name(name: &str) -> String {
+    name.trim_end_matches('_').to_string()
+}
+
 fn wrap_render_block(name: &str, describe: &str, term_width: Option<usize>) -> String {
     let size = term_width.unwrap_or(999) - name.len();
     let empty = " ".repeat(name.len());
