@@ -7,7 +7,7 @@ use std::{
 use crate::{
     command::Command,
     compgen::CompColor,
-    param::{FlagOptionParam, PositionalParam},
+    param::{ChoiceData, FlagOptionParam, ParamData, PositionalParam},
     utils::run_param_fns,
     ArgcValue, Shell,
 };
@@ -22,15 +22,16 @@ pub(crate) struct Matcher<'a, 'b> {
     args: &'b [String],
     flag_option_args: Vec<Vec<FlagOptionArg<'a, 'b>>>,
     positional_args: Vec<&'b str>,
-    dashes: Vec<usize>,
+    dashes: Option<usize>,
     arg_comp: ArgComp,
-    choices_fns: HashSet<&'a str>,
+    choice_fns: HashSet<&'a str>,
     script_path: Option<String>,
     term_width: Option<usize>,
-    is_last_arg_option_assign: bool,
+    split_last_arg_at: Option<usize>,
 }
 
 type FlagOptionArg<'a, 'b> = (&'b str, Vec<&'b str>, Option<&'a str>);
+type LevelCommand<'a, 'b> = (&'b str, &'a Command, String, usize);
 
 #[derive(Debug, PartialEq, Eq)]
 pub(crate) enum ArgComp {
@@ -63,20 +64,20 @@ pub(crate) enum MatchError {
 
 impl<'a, 'b> Matcher<'a, 'b> {
     pub(crate) fn new(root: &'a Command, args: &'b [String]) -> Self {
-        let mut cmds = vec![(args[0].as_str(), root, args[0].clone(), 0)];
+        let mut cmds: Vec<LevelCommand> = vec![(args[0].as_str(), root, args[0].clone(), 0)];
         let mut cmd_level = 0;
         let mut arg_index = 1;
         let mut flag_option_args = vec![vec![]];
         let mut positional_args = vec![];
-        let mut dashes = vec![];
-        let mut is_last_arg_option_assign = false;
+        let mut dashes = None;
+        let mut split_last_arg_at = None;
         let mut arg_comp = ArgComp::Any;
-        let mut choices_fns = HashSet::new();
+        let mut choice_fns = HashSet::new();
         let args_len = args.len();
         if root.delegated() {
             positional_args = args.iter().skip(1).map(|v| v.as_str()).collect();
         } else {
-            let mut is_rest_args_positional = false;
+            let mut is_rest_args_positional = false; // option(e.g. -f --foo) will be treated as positional arg
             if let Some(arg) = args.last() {
                 if arg.starts_with('-') {
                     arg_comp = ArgComp::FlagOrOption;
@@ -88,7 +89,6 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 let cmd = cmds[cmd_level].1;
                 let arg = args[arg_index].as_str();
                 if arg == "--" {
-                    dashes.push(positional_args.len());
                     if is_rest_args_positional {
                         add_positional_arg(
                             &mut positional_args,
@@ -97,8 +97,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             cmd,
                         );
                     }
+                    if dashes.is_none() {
+                        dashes = Some(positional_args.len());
+                        if arg_index != args_len - 1 {
+                            is_rest_args_positional = true
+                        }
+                    }
                 } else if is_rest_args_positional
-                    || !dashes.is_empty()
                     || (cmd.no_flags_options_subcommands() && !KNOWN_OPTIONS.contains(&arg))
                 {
                     add_positional_arg(
@@ -109,66 +114,97 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     );
                 } else if arg.starts_with('-') {
                     if let Some((k, v)) = arg.split_once('=') {
-                        let param = cmd.find_flag_option(k);
-                        if arg_index == args_len - 1 {
-                            if let Some(param) = param {
-                                arg_comp = ArgComp::OptionValue(param.name.clone(), 0)
+                        if let Some(param) = cmd.find_flag_option(k) {
+                            add_param_choice_fn(&mut choice_fns, param);
+                            if arg_index == args_len - 1 {
+                                arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
+                                split_last_arg_at = Some(k.len() + 1);
                             }
-                            is_last_arg_option_assign = true;
+                            flag_option_args[cmd_level].push((k, vec![v], Some(param.name())));
+                        } else if let Some((param, prefix)) = cmd.find_prefixed_option(arg) {
+                            add_param_choice_fn(&mut choice_fns, param);
+                            match_prefix_option(
+                                &mut flag_option_args[cmd_level],
+                                args,
+                                &mut arg_index,
+                                param,
+                                &mut arg_comp,
+                                &mut split_last_arg_at,
+                                &prefix,
+                            );
+                        } else {
+                            flag_option_args[cmd_level].push((k, vec![v], None));
                         }
-                        if let Some((choices_fn, validate)) =
-                            param.and_then(|v| v.choices_fn.as_ref())
-                        {
-                            if *validate {
-                                choices_fns.insert(choices_fn.as_str());
-                            }
-                        }
-                        flag_option_args[cmd_level].push((
-                            k,
-                            vec![v],
-                            param.map(|v| v.name.as_str()),
-                        ));
                     } else if let Some(param) = cmd.find_flag_option(arg) {
-                        if let Some((choices_fn, validate)) = param.choices_fn.as_ref() {
-                            if *validate {
-                                choices_fns.insert(choices_fn.as_str());
-                            }
-                        }
+                        add_param_choice_fn(&mut choice_fns, param);
                         match_flag_option(
                             &mut flag_option_args[cmd_level],
                             args,
                             &mut arg_index,
                             param,
                             &mut arg_comp,
+                            &mut split_last_arg_at,
                         );
-                    } else if let Some(mut list) = match_combine_shorts(cmd, arg) {
-                        let name = list.pop().and_then(|v| v.2).unwrap();
-                        let param = cmd.find_flag_option(name).unwrap();
-                        if let Some((choices_fn, validate)) = param.choices_fn.as_ref() {
-                            if *validate {
-                                choices_fns.insert(choices_fn.as_str());
-                            }
+                    } else if let Some((param, prefix)) = cmd.find_prefixed_option(arg) {
+                        add_param_choice_fn(&mut choice_fns, param);
+                        match_prefix_option(
+                            &mut flag_option_args[cmd_level],
+                            args,
+                            &mut arg_index,
+                            param,
+                            &mut arg_comp,
+                            &mut split_last_arg_at,
+                            &prefix,
+                        );
+                    } else if let Some(subcmd) = find_subcommand(cmd, arg, &positional_args) {
+                        match_command(
+                            &mut cmds,
+                            &mut cmd_level,
+                            &mut flag_option_args,
+                            subcmd,
+                            arg,
+                            arg_index,
+                        );
+                        if arg_index == args_len - 1 && arg.len() == 2 {
+                            arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
                         }
-                        flag_option_args[cmd_level].extend(list);
+                    } else if let Some((mut arr, maybe_subcmd)) = match_combine_shorts(cmd, arg) {
+                        let mut current_cmd = cmd;
+                        if let Some(subcmd) = maybe_subcmd {
+                            match_command(
+                                &mut cmds,
+                                &mut cmd_level,
+                                &mut flag_option_args,
+                                subcmd,
+                                arg,
+                                arg_index,
+                            );
+                            current_cmd = subcmd;
+                        }
+                        let name = arr.pop().and_then(|v| v.2).unwrap();
+                        let param = current_cmd.find_flag_option(name).unwrap();
+                        add_param_choice_fn(&mut choice_fns, param);
+                        flag_option_args[cmd_level].extend(arr);
                         match_flag_option(
                             &mut flag_option_args[cmd_level],
                             args,
                             &mut arg_index,
                             param,
                             &mut arg_comp,
+                            &mut split_last_arg_at,
                         );
                     } else {
                         flag_option_args[cmd_level].push((arg, vec![], None));
                     }
-                } else if let Some(subcmd) = cmd.find_subcommand(arg) {
-                    cmd_level += 1;
-                    cmds.push((
-                        arg,
+                } else if let Some(subcmd) = find_subcommand(cmd, arg, &positional_args) {
+                    match_command(
+                        &mut cmds,
+                        &mut cmd_level,
+                        &mut flag_option_args,
                         subcmd,
-                        subcmd.name.clone().unwrap_or_else(|| arg.to_string()),
+                        arg,
                         arg_index,
-                    ));
-                    flag_option_args.push(vec![]);
+                    );
                 } else {
                     add_positional_arg(
                         &mut positional_args,
@@ -185,10 +221,10 @@ impl<'a, 'b> Matcher<'a, 'b> {
             }
 
             let last_cmd = cmds.last().unwrap().1;
-            choices_fns.extend(last_cmd.positional_params.iter().filter_map(|v| {
-                if let Some((choices_fn, validate)) = v.choices_fn.as_ref() {
+            choice_fns.extend(last_cmd.positional_params.iter().filter_map(|v| {
+                if let Some((choice_fn, validate)) = v.choice_fn() {
                     if *validate {
-                        return Some(choices_fn.as_str());
+                        return Some(choice_fn.as_str());
                     }
                 }
                 None
@@ -201,10 +237,10 @@ impl<'a, 'b> Matcher<'a, 'b> {
             positional_args,
             dashes,
             arg_comp,
-            choices_fns,
+            choice_fns,
             script_path: None,
             term_width: None,
-            is_last_arg_option_assign,
+            split_last_arg_at,
         }
     }
 
@@ -235,11 +271,12 @@ impl<'a, 'b> Matcher<'a, 'b> {
 
     pub(crate) fn to_arg_values_for_choice_fn(&self) -> Vec<ArgcValue> {
         let mut output: Vec<ArgcValue> = self.to_arg_values_base();
-        if !self.dashes.is_empty() {
-            output.push(ArgcValue::Multiple(
-                "_dashes".into(),
-                self.dashes.iter().map(|v| v.to_string()).collect(),
-            ));
+        if let Some(idx) = self.dashes {
+            output.push(ArgcValue::Single("_dashes".into(), idx.to_string()));
+        }
+        let last_cmd = self.cmds[self.cmds.len() - 1].1;
+        if let Some(name) = &last_cmd.name {
+            output.push(ArgcValue::Single("_cmd_fn".into(), name.to_string()));
         }
         output
     }
@@ -269,19 +306,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let mut output = match &self.arg_comp {
             ArgComp::FlagOrOption => {
                 let mut output = self.comp_flag_options();
-                if let Some((value, param)) = self
-                    .args
-                    .last()
-                    .and_then(|value| last_cmd.find_flag_option(value).map(|param| (value, param)))
-                {
-                    let describe = param.describe_head();
-                    let kind = if param.is_flag() {
-                        CompColor::of_flag()
-                    } else {
-                        CompColor::of_option()
-                    };
-                    output.push((value.clone(), describe.into(), kind));
-                }
+                output.extend(comp_subcomands(last_cmd, true));
                 output
             }
             ArgComp::FlagOrOptionCombine(value) => {
@@ -289,7 +314,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                     .comp_flag_options()
                     .iter()
                     .filter_map(|(x, y, z)| {
-                        if x.len() == 2 {
+                        if x.len() == 2 && x != value {
                             Some((format!("{value}{}", &x[1..]), y.to_string(), *z))
                         } else {
                             None
@@ -303,7 +328,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
             }
             ArgComp::CommandOrPositional => {
                 if self.positional_args.len() == 2 && self.positional_args[0] == "help" {
-                    return comp_subcomands(last_cmd);
+                    return comp_subcomands(last_cmd, false);
                 }
                 if level > 0
                     && self.positional_args.is_empty()
@@ -315,7 +340,11 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 comp_subcommands_positional(last_cmd, &values, self.positional_args.len() < 2)
             }
             ArgComp::OptionValue(name, index) => {
-                if let Some(param) = last_cmd.flag_option_params.iter().find(|v| &v.name == name) {
+                if let Some(param) = last_cmd
+                    .flag_option_params
+                    .iter()
+                    .find(|v| v.name() == name)
+                {
                     comp_flag_option(param, *index)
                 } else {
                     vec![]
@@ -323,7 +352,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
             }
             ArgComp::Any => {
                 if self.positional_args.len() == 2 && self.positional_args[0] == "help" {
-                    return comp_subcomands(last_cmd);
+                    return comp_subcomands(last_cmd, false);
                 }
                 let values = self.match_positionals();
                 comp_subcommands_positional(last_cmd, &values, self.positional_args.len() < 2)
@@ -342,8 +371,8 @@ impl<'a, 'b> Matcher<'a, 'b> {
         output
     }
 
-    pub(crate) fn is_last_arg_option_assign(&self) -> bool {
-        self.is_last_arg_option_assign
+    pub(crate) fn split_last_arg_at(&self) -> Option<usize> {
+        self.split_last_arg_at
     }
 
     fn to_arg_values_base(&self) -> Vec<ArgcValue> {
@@ -351,7 +380,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let cmds_len = self.cmds.len();
         let level = cmds_len - 1;
         let last_cmd = self.cmds[level].1;
-        let args_index = self.cmds[level].3;
+        let cmd_arg_index = self.cmds[level].3;
         for level in 0..cmds_len {
             let args = &self.flag_option_args[level];
             let cmd = self.cmds[level].1;
@@ -359,7 +388,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 let values: Vec<&[&str]> = args
                     .iter()
                     .filter_map(|(_, value, name)| {
-                        if let Some(true) = name.map(|v| param.name == v) {
+                        if let Some(true) = name.map(|v| param.name() == v) {
                             Some(value.as_slice())
                         } else {
                             None
@@ -383,7 +412,10 @@ impl<'a, 'b> Matcher<'a, 'b> {
             }
         }
         output.push(ArgcValue::Multiple("_args".into(), self.args.to_vec()));
-        output.push(ArgcValue::Single("_index".into(), args_index.to_string()));
+        output.push(ArgcValue::Single(
+            "_cmd_arg_index".into(),
+            cmd_arg_index.to_string(),
+        ));
         output
     }
 
@@ -456,7 +488,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let mut missing_params: Vec<String> = if positional_params_len > positional_values_len {
             last_cmd.positional_params[positional_values_len..]
                 .iter()
-                .filter(|param| param.required)
+                .filter(|param| param.required())
                 .map(|v| v.render_value())
                 .collect()
         } else {
@@ -468,7 +500,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         for (i, param) in last_cmd.positional_params.iter().enumerate() {
             if let (Some(values), Some(choices)) = (
                 positional_values.get(i),
-                get_param_choices(&param.choices, &param.choices_fn, &choices_fn_values),
+                get_param_choice(&param.data.choice, &choices_fn_values),
             ) {
                 for value in values.iter() {
                     if !choices.contains(&value.to_string()) {
@@ -489,8 +521,8 @@ impl<'a, 'b> Matcher<'a, 'b> {
             let mut missing_flag_options: IndexSet<&str> = cmd
                 .flag_option_params
                 .iter()
-                .filter(|v| v.required)
-                .map(|v| v.name.as_str())
+                .filter(|v| v.required())
+                .map(|v| v.name())
                 .collect();
             for (i, (key, _, name)) in args.iter().enumerate() {
                 match *name {
@@ -509,10 +541,10 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 missing_params.extend(missing_flag_options)
             }
             for (name, indexes) in flag_option_map {
-                if let Some(param) = cmd.flag_option_params.iter().find(|v| v.name == name) {
+                if let Some(param) = cmd.flag_option_params.iter().find(|v| v.name() == name) {
                     let values_list: Vec<&[&str]> =
                         indexes.iter().map(|v| args[*v].1.as_slice()).collect();
-                    if !param.multiple && values_list.len() > 1 {
+                    if !param.multiple() && values_list.len() > 1 {
                         return Some(MatchError::NotMultipleArgument(level, param.render_name()));
                     }
                     for values in values_list.iter() {
@@ -523,7 +555,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                                     param.render_name(),
                                     values[0].to_string(),
                                 ));
-                            } else if !param.multiple {
+                            } else if !param.multiple() {
                                 return Some(MatchError::MismatchValues(
                                     level,
                                     param.render_name_values(),
@@ -531,7 +563,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             }
                         }
                         if let Some(choices) =
-                            get_param_choices(&param.choices, &param.choices_fn, &choices_fn_values)
+                            get_param_choice(&param.data.choice, &choices_fn_values)
                         {
                             for value in values.iter() {
                                 if !choices.contains(&value.to_string()) {
@@ -564,7 +596,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
     fn run_choices_fns(&'a self) -> Option<HashMap<&'a str, Vec<String>>> {
         let script_path = self.script_path.as_ref()?;
         let mut choices_fn_values = HashMap::new();
-        let fns: Vec<&str> = self.choices_fns.iter().copied().collect();
+        let fns: Vec<&str> = self.choice_fns.iter().copied().collect();
         let mut envs = HashMap::new();
         envs.insert("ARGC_OS".into(), env::consts::OS.to_string());
         envs.insert("ARGC_PATH_SEP".into(), MAIN_SEPARATOR.into());
@@ -598,14 +630,14 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let mut param_index = 0;
         while param_index < params_len && arg_index < args_len {
             let param = &cmd.positional_params[param_index];
-            if param.multiple {
-                let dash_idx = self.dashes.first().cloned().unwrap_or_default();
+            if param.multiple() {
+                let dashes_at = self.dashes.unwrap_or_default();
                 let takes = if param_index == 0
-                    && dash_idx > 0
+                    && dashes_at > 0
                     && params_len == 2
-                    && cmd.positional_params[1].multiple
+                    && cmd.positional_params[1].multiple()
                 {
-                    dash_idx
+                    dashes_at
                 } else {
                     (args_len - arg_index).saturating_sub(params_len - param_index) + 1
                 };
@@ -763,9 +795,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
             .iter()
             .filter_map(|v| v.2)
             .collect();
+        let last = self.args.last().map(|v| v.as_str()).unwrap_or_default();
         for param in cmd.flag_option_params.iter() {
-            let exist = args.contains(param.name.as_str());
-            if !exist || param.multiple {
+            let mut exist = args.contains(param.name());
+            if !last.is_empty() && param.is_match(last) {
+                exist = false;
+            }
+            if !exist || param.multiple() {
                 let describe = param.describe_head();
                 let kind = if param.is_flag() {
                     CompColor::of_flag()
@@ -783,6 +819,20 @@ impl<'a, 'b> Matcher<'a, 'b> {
 
 pub(crate) type CompItem = (String, String, CompColor);
 
+fn find_subcommand<'a>(
+    cmd: &'a Command,
+    arg: &str,
+    positional_args: &Vec<&str>,
+) -> Option<&'a Command> {
+    cmd.find_subcommand(arg).and_then(|v| {
+        if positional_args.is_empty() {
+            Some(v)
+        } else {
+            None
+        }
+    })
+}
+
 fn add_positional_arg<'a>(
     positional_args: &mut Vec<&'a str>,
     arg: &'a str,
@@ -791,7 +841,7 @@ fn add_positional_arg<'a>(
 ) {
     positional_args.push(arg);
     if !*is_rest_args_positional
-        && cmd.positional_params.last().map(|v| v.terminated) == Some(true)
+        && cmd.positional_params.last().map(|v| v.terminated()) == Some(true)
         && positional_args.len() >= cmd.positional_params.len() - 1
     {
         *is_rest_args_positional = true;
@@ -816,39 +866,49 @@ fn take_value_args(args: &[String], start: usize, len: usize) -> Vec<&str> {
 fn match_combine_shorts<'a, 'b>(
     cmd: &'a Command,
     arg: &'b str,
-) -> Option<Vec<FlagOptionArg<'a, 'b>>> {
-    if arg.len() > 2 && !arg.starts_with("--") {
-        let mut output = vec![];
-        for ch in arg.chars().skip(1) {
-            let name: String = format!("-{ch}");
-            if let Some(param) = cmd.find_flag_option(&name) {
-                output.push((arg, vec![], Some(param.name.as_str())))
-            } else {
-                return None;
+) -> Option<(Vec<FlagOptionArg<'a, 'b>>, Option<&'a Command>)> {
+    if arg.starts_with("--") || arg.len() <= 2 {
+        return None;
+    }
+    let mut current_cmd = cmd;
+    let mut subcmd = None;
+    let mut output = vec![];
+    for (i, ch) in arg.chars().skip(1).enumerate() {
+        let name: String = format!("-{ch}");
+        if i == 0 {
+            if let Some(v) = cmd.find_subcommand(&name) {
+                current_cmd = v;
+                subcmd = Some(v);
+                continue;
             }
         }
-        Some(output)
-    } else {
-        None
+        if let Some(param) = current_cmd.find_flag_option(&name) {
+            output.push((arg, vec![], Some(param.name())))
+        } else {
+            return None;
+        }
     }
+
+    Some((output, subcmd))
 }
 
 fn match_flag_option<'a, 'b>(
-    output: &mut Vec<FlagOptionArg<'a, 'b>>,
+    flag_option_args: &mut Vec<FlagOptionArg<'a, 'b>>,
     args: &'b [String],
     arg_index: &mut usize,
     param: &'a FlagOptionParam,
     arg_comp: &mut ArgComp,
+    split_last_arg_at: &mut Option<usize>,
 ) {
-    if param.terminated {
+    if param.terminated() {
         let value_args: Vec<&str> = args[*arg_index + 1..].iter().map(|v| v.as_str()).collect();
         let arg = &args[*arg_index];
         *arg_index += value_args.len();
         if !value_args.is_empty() {
             *arg_comp =
-                ArgComp::OptionValue(param.name.clone(), value_args.len().saturating_sub(1));
+                ArgComp::OptionValue(param.name().to_string(), value_args.len().saturating_sub(1));
         }
-        output.push((arg, value_args, Some(param.name.as_str())));
+        flag_option_args.push((arg, value_args, Some(param.name())));
     } else {
         let values_len = param.values_size();
         let args_len = args.len();
@@ -856,20 +916,63 @@ fn match_flag_option<'a, 'b>(
         let arg = &args[*arg_index];
         *arg_index += value_args.len();
         if *arg_index == args_len - 1 {
-            if *arg_comp != ArgComp::FlagOrOption
-                && param.is_option()
-                && value_args.len() <= values_len
-            {
-                *arg_comp =
-                    ArgComp::OptionValue(param.name.clone(), value_args.len().saturating_sub(1));
-            } else if *arg_comp == ArgComp::FlagOrOption
-                && param.is_flag()
-                && !(arg.len() > 2 && param.is_match(arg))
-            {
+            if *arg_comp != ArgComp::FlagOrOption {
+                if param.is_option() && value_args.len() <= values_len {
+                    *arg_comp = ArgComp::OptionValue(
+                        param.name().to_string(),
+                        value_args.len().saturating_sub(1),
+                    );
+                }
+            } else if let Some(prefix) = param.prefixed() {
+                if arg.starts_with(&prefix) {
+                    *arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
+                    *split_last_arg_at = Some(prefix.len());
+                }
+            } else if param.is_flag() && !(arg.len() > 2 && param.is_match(arg)) {
                 *arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
             }
         }
-        output.push((arg, value_args, Some(param.name.as_str())));
+        flag_option_args.push((arg, value_args, Some(param.name())));
+    }
+}
+
+fn match_prefix_option<'a, 'b>(
+    flag_option_args: &mut Vec<FlagOptionArg<'a, 'b>>,
+    args: &'b [String],
+    arg_index: &mut usize,
+    param: &'a FlagOptionParam,
+    arg_comp: &mut ArgComp,
+    split_last_arg_at: &mut Option<usize>,
+    prefix: &str,
+) {
+    let prefix_len = prefix.len();
+    let args_len = args.len();
+    let arg = &args[*arg_index];
+    if *arg_index == args_len - 1 {
+        *arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
+        *split_last_arg_at = Some(prefix_len);
+    }
+    flag_option_args.push((arg, vec![&arg[prefix_len..]], Some(param.name())));
+}
+
+fn match_command<'a, 'b>(
+    cmds: &mut Vec<LevelCommand<'a, 'b>>,
+    cmd_level: &mut usize,
+    flag_option_args: &mut Vec<Vec<FlagOptionArg<'a, 'b>>>,
+    subcmd: &'a Command,
+    arg: &'b str,
+    arg_index: usize,
+) {
+    *cmd_level += 1;
+    cmds.push((arg, subcmd, arg.to_string(), arg_index));
+    flag_option_args.push(vec![]);
+}
+
+fn add_param_choice_fn<'a>(choice_fns: &mut HashSet<&'a str>, param: &'a FlagOptionParam) {
+    if let Some((choice_fn, validate)) = param.choice_fn() {
+        if *validate {
+            choice_fns.insert(choice_fn.as_str());
+        }
     }
 }
 
@@ -880,7 +983,7 @@ fn comp_subcommands_positional(
 ) -> Vec<CompItem> {
     let mut output = vec![];
     if with_subcmd {
-        output.extend(comp_subcomands(cmd))
+        output.extend(comp_subcomands(cmd, false))
     }
     if values.is_empty() || values.len() > cmd.positional_params.len() {
         return output;
@@ -889,14 +992,36 @@ fn comp_subcommands_positional(
     output
 }
 
-fn comp_subcomands(cmd: &Command) -> Vec<CompItem> {
+fn comp_subcomands(cmd: &Command, flag: bool) -> Vec<CompItem> {
     let mut output = vec![];
+    let mut has_help_subcmd = false;
+    let mut describe_help_subcmd = false;
     for subcmd in cmd.subcommands.iter() {
         let describe = subcmd.describe_head();
         for v in subcmd.list_names() {
-            output.push((v, describe.to_string(), CompColor::of_command()))
+            if (flag && v.starts_with('-')) || (!flag && !v.starts_with('-')) {
+                if !flag {
+                    has_help_subcmd = true;
+                }
+                if !describe.is_empty() {
+                    describe_help_subcmd = true
+                }
+                output.push((v, describe.to_string(), CompColor::of_command()))
+            }
         }
     }
+    if has_help_subcmd {
+        let describe = if describe_help_subcmd {
+            "Show help for a command"
+        } else {
+            ""
+        };
+        output.push((
+            "help".to_string(),
+            describe.to_string(),
+            CompColor::of_command(),
+        ))
+    }
     output
 }
 
@@ -906,39 +1031,21 @@ fn comp_flag_option(param: &FlagOptionParam, index: usize) -> Vec<CompItem> {
         .get(index)
         .map(|v| v.as_str())
         .unwrap_or_else(|| param.arg_value_names.last().unwrap());
-    comp_param(
-        param.describe_head(),
-        value_name,
-        &param.choices,
-        &param.choices_fn,
-        &param.multi_char,
-    )
+    comp_param(param.describe_head(), value_name, &param.data)
 }
 
 fn comp_positional(param: &PositionalParam) -> Vec<CompItem> {
-    comp_param(
-        param.describe_head(),
-        &param.arg_value_name,
-        &param.choices,
-        &param.choices_fn,
-        &param.multi_char,
-    )
+    comp_param(param.describe_head(), &param.arg_value_name, &param.data)
 }
 
-fn comp_param(
-    describe: &str,
-    value_name: &str,
-    choices: &Option<Vec<String>>,
-    choices_fn: &Option<(String, bool)>,
-    multi_char: &Option<char>,
-) -> Vec<CompItem> {
-    let choices: Option<Either<Vec<String>, String>> = if let Some(choices_fn) = choices_fn {
-        Some(Either::Right(choices_fn.0.to_string()))
-    } else {
-        choices
-            .as_ref()
-            .map(|choices| Either::Left(choices.iter().map(|v| v.to_string()).collect()))
-    };
+fn comp_param(describe: &str, value_name: &str, data: &ParamData) -> Vec<CompItem> {
+    let choices: Option<Either<Vec<String>, String>> =
+        if let Some((choice_fn, _)) = data.choice_fn() {
+            Some(Either::Right(choice_fn.to_string()))
+        } else {
+            data.choice_values()
+                .map(|choices| Either::Left(choices.iter().map(|v| v.to_string()).collect()))
+        };
     let mut output = if let Some(choices) = choices {
         match choices {
             Either::Left(choices) => choices
@@ -955,7 +1062,7 @@ fn comp_param(
         let value = format!("__argc_value={}", value_name);
         vec![(value, describe.into(), CompColor::of_value())]
     };
-    if let Some(ch) = multi_char {
+    if let Some(ch) = data.multi_char() {
         output.insert(
             0,
             (
@@ -968,18 +1075,19 @@ fn comp_param(
     output
 }
 
-fn get_param_choices<'a, 'b: 'a>(
-    choices: &'a Option<Vec<String>>,
-    choices_fn: &'a Option<(String, bool)>,
+fn get_param_choice<'a, 'b: 'a>(
+    choice: &'a Option<ChoiceData>,
     choices_fn_values: &'a HashMap<&str, Vec<String>>,
 ) -> Option<&'a Vec<String>> {
-    choices.as_ref().or_else(|| {
-        choices_fn.as_ref().and_then(|(fn_name, validate)| {
+    match choice {
+        Some(ChoiceData::Values(v)) => Some(v),
+        Some(ChoiceData::Fn(choice_fn, validate)) => {
             if *validate {
-                choices_fn_values.get(fn_name.as_str())
+                choices_fn_values.get(choice_fn.as_str())
             } else {
                 None
             }
-        })
-    })
+        }
+        None => None,
+    }
 }
