@@ -1,4 +1,6 @@
-use crate::param::{FlagOptionParam, ParamData, PositionalParam};
+use crate::param::{
+    ChoiceData, DefaultData, FlagOptionParam, Modifier, ParamData, PositionalParam,
+};
 use crate::utils::{is_choice_value_terminate, is_default_value_terminate};
 use crate::Result;
 use anyhow::bail;
@@ -44,7 +46,7 @@ pub(crate) enum EventData {
     Unknown(String),
 }
 
-#[derive(PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq)]
 pub(crate) enum EventScope {
     Root,
     CmdStart,
@@ -216,8 +218,15 @@ fn parse_with_long_option_param(input: &str) -> nom::IResult<&str, FlagOptionPar
             parse_zero_or_many_value_notations,
             parse_tail,
         )),
-        |(short, dashes, arg, value_names, describe)| {
-            FlagOptionParam::new(arg, describe, short, false, dashes, &value_names)
+        |(short, hyphens, arg, value_names, describe)| {
+            FlagOptionParam::new(
+                arg,
+                describe,
+                short,
+                false,
+                hyphens.len() == 1,
+                &value_names,
+            )
         },
     )(input)
 }
@@ -244,8 +253,7 @@ fn parse_no_long_option_param(input: &str) -> nom::IResult<&str, FlagOptionParam
             parse_tail,
         )),
         |(arg, value_names, describe)| {
-            let short = arg.name.chars().next();
-            FlagOptionParam::new(arg, describe, short, false, "", &value_names)
+            FlagOptionParam::new(arg, describe, None, false, true, &value_names)
         },
     )(input)
 }
@@ -273,6 +281,7 @@ fn parse_positional_param(input: &str) -> nom::IResult<&str, PositionalParam> {
 fn parse_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     alt((parse_with_long_flag_param, parse_no_long_flag_param))(input)
 }
+
 // Parse `@flag`
 fn parse_with_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam> {
     map(
@@ -282,8 +291,8 @@ fn parse_with_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam
             parse_long_flag_and_asterisk,
             parse_tail,
         )),
-        |(short, dashes, arg, describe)| {
-            FlagOptionParam::new(arg, describe, short, true, dashes, &[])
+        |(short, hyphens, arg, describe)| {
+            FlagOptionParam::new(arg, describe, short, true, hyphens.len() == 1, &[])
         },
     )(input)
 }
@@ -295,10 +304,7 @@ fn parse_no_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam>
             preceded(pair(space0, tag("-")), parse_short_flag_and_asterisk),
             parse_tail,
         )),
-        |(arg, describe)| {
-            let short = arg.name.chars().next();
-            FlagOptionParam::new(arg, describe, short, true, "", &[])
-        },
+        |(arg, describe)| FlagOptionParam::new(arg, describe, None, true, true, &[]),
     )(input)
 }
 
@@ -306,7 +312,7 @@ fn parse_no_long_flag_param(input: &str) -> nom::IResult<&str, FlagOptionParam>
 fn parse_long_flag_and_asterisk(input: &str) -> nom::IResult<&str, ParamData> {
     alt((
         map(terminated(parse_param_name, tag("*")), |mut arg| {
-            arg.multiple = true;
+            arg.modifer = Modifier::MultipleOptional;
             arg
         }),
         parse_param_name,
@@ -321,41 +327,53 @@ fn parse_short_flag_and_asterisk(input: &str) -> nom::IResult<&str, ParamData> {
         })(input)
     }
     map(pair(parser, opt(tag("*"))), |(mut arg, multiple)| {
-        arg.multiple = multiple.is_some();
+        if multiple.is_some() {
+            arg.modifer = Modifier::MultipleOptional;
+        }
         arg
     })(input)
 }
 
-// Parse `str!` `str*` `str+` `str`
+// Parse `str!` `str~` `str*` `str+` `str`
 fn parse_param_modifer(input: &str) -> nom::IResult<&str, ParamData> {
     alt((
         map(terminated(parse_param_name, tag("!")), |mut arg| {
-            arg.required = true;
+            arg.modifer = Modifier::Required;
             arg
         }),
         map(terminated(parse_param_name, tag("~")), |mut arg| {
-            arg.multiple = true;
-            arg.terminated = true;
+            arg.modifer = Modifier::Terminated;
             arg
         }),
         map(
             pair(parse_param_name, preceded(tag("*"), opt(parse_multi_char))),
             |(mut arg, multi_char)| {
-                arg.multiple = true;
-                arg.multi_char = multi_char;
+                let modifier = match multi_char {
+                    Some(c) => Modifier::MultiCharOptional(c),
+                    None => Modifier::MultipleOptional,
+                };
+                arg.modifer = modifier;
                 arg
             },
         ),
         map(
             pair(parse_param_name, preceded(tag("+"), opt(parse_multi_char))),
             |(mut arg, multi_char)| {
-                arg.required = true;
-                arg.multiple = true;
-                arg.multi_char = multi_char;
+                let modifier = match multi_char {
+                    Some(c) => Modifier::MultiCharRequired(c),
+                    None => Modifier::MultipleRequired,
+                };
+                arg.modifer = modifier;
                 arg
             },
         ),
-        parse_param_name,
+        map(parse_param_name, |mut arg| {
+            if let Some(name) = arg.name.strip_suffix('-') {
+                arg.name = name.to_string();
+                arg.modifer = Modifier::Prefixed;
+            }
+            arg
+        }),
     ))(input)
 }
 
@@ -364,7 +382,7 @@ fn parse_param_assign(input: &str) -> nom::IResult<&str, ParamData> {
     map(
         separated_pair(parse_param_name, char('='), parse_default_value),
         |(mut arg, value)| {
-            arg.default = Some(value.to_string());
+            arg.default = Some(DefaultData::Value(value.to_string()));
             arg
         },
     )(input)
@@ -374,8 +392,8 @@ fn parse_param_assign(input: &str) -> nom::IResult<&str, ParamData> {
 fn parse_param_assign_fn(input: &str) -> nom::IResult<&str, ParamData> {
     map(
         separated_pair(parse_param_name, char('='), parse_value_fn),
-        |(mut arg, value)| {
-            arg.default_fn = Some(value.to_string());
+        |(mut arg, f)| {
+            arg.default = Some(DefaultData::Fn(f.to_string()));
             arg
         },
     )(input)
@@ -387,10 +405,13 @@ fn parse_param_modifer_choices_default(input: &str) -> nom::IResult<&str, ParamD
             parse_param_modifer,
             delimited(char('['), parse_choices_default, char(']')),
         ),
-        |(mut arg, (choices, default))| {
-            arg.choices = Some(choices.iter().map(|v| v.to_string()).collect());
-            arg.required = false;
-            arg.default = default.map(|v| v.to_string());
+        |(mut arg, (values, default))| {
+            arg.choice = Some(ChoiceData::Values(
+                values.iter().map(|v| v.to_string()).collect(),
+            ));
+            if let Some(value) = default {
+                arg.default = Some(DefaultData::Value(value.to_string()));
+            }
             arg
         },
     )(input)
@@ -402,8 +423,10 @@ fn parse_param_modifer_choices(input: &str) -> nom::IResult<&str, ParamData> {
             parse_param_modifer,
             delimited(char('['), parse_choices, char(']')),
         ),
-        |(mut arg, choices)| {
-            arg.choices = Some(choices.iter().map(|v| v.to_string()).collect());
+        |(mut arg, values)| {
+            arg.choice = Some(ChoiceData::Values(
+                values.iter().map(|v| v.to_string()).collect(),
+            ));
             arg
         },
     )(input)
@@ -415,8 +438,8 @@ fn parse_param_modifer_choices_fn(input: &str) -> nom::IResult<&str, ParamData>
             parse_param_modifer,
             delimited(char('['), pair(opt(char('?')), parse_value_fn), char(']')),
         ),
-        |(mut arg, (validate, choices_fn))| {
-            arg.choices_fn = Some((choices_fn.into(), validate.is_none()));
+        |(mut arg, (validate, f))| {
+            arg.choice = Some(ChoiceData::Fn(f.into(), validate.is_none()));
             arg
         },
     )(input)
@@ -483,7 +506,28 @@ fn parse_name_list(input: &str) -> nom::IResult<&str, Vec<&str>> {
 }
 
 fn parse_fn_name(input: &str) -> nom::IResult<&str, &str> {
-    take_while1(is_not_fn_name_char)(input)
+    take_while1(|c| {
+        !matches!(
+            c,
+            ' ' | '\t'
+                | '"'
+                | '\''
+                | '`'
+                | '('
+                | ')'
+                | '['
+                | ']'
+                | '{'
+                | '}'
+                | '<'
+                | '>'
+                | '$'
+                | '&'
+                | '\\'
+                | ';'
+                | '|'
+        )
+    })(input)
 }
 
 fn parse_name(input: &str) -> nom::IResult<&str, &str> {
@@ -582,35 +626,16 @@ fn create_err(input: &str, kind: ErrorKind) -> nom::Err<nom::error::Error<&str>>
     nom::Err::Error(nom::error::Error::new(input, kind))
 }
 
-fn is_not_fn_name_char(c: char) -> bool {
-    !matches!(
-        c,
-        ' ' | '\t'
-            | '"'
-            | '\''
-            | '`'
-            | '('
-            | ')'
-            | '['
-            | ']'
-            | '{'
-            | '}'
-            | '<'
-            | '>'
-            | '$'
-            | '&'
-            | '\\'
-            | ';'
-            | '|'
-    )
-}
-
 fn is_name_char(c: char) -> bool {
-    c.is_ascii_alphanumeric() || matches!(c, '_' | '-' | '.')
+    c.is_ascii_alphanumeric() || matches!(c, '_' | '-' | '.' | ':')
 }
 
 fn is_short_char(c: char) -> bool {
-    c.is_ascii() && is_not_fn_name_char(c) && !matches!(c, '-')
+    c.is_ascii()
+        && !matches!(
+            c,
+            '-' | '\t' | '"' | '\'' | '(' | ')' | '[' | ']' | '<' | '>' | '&' | '\\' | ';' | '|'
+        )
 }
 
 fn take_comment_lines(lines: &[&str], idx: usize, output: &mut String) -> usize {
@@ -706,6 +731,8 @@ mod tests {
     #[test]
     fn test_parse_with_long_option_arg() {
         assert_parse_option_arg!("-f --foo=a <FOO> A foo option");
+        assert_parse_option_arg!("--foo:bar");
+        assert_parse_option_arg!("--foo.bar");
         assert_parse_option_arg!("--foo!");
         assert_parse_option_arg!("--foo+");
         assert_parse_option_arg!("--foo*");
@@ -719,10 +746,10 @@ mod tests {
         assert_parse_option_arg!("--foo[`_foo`]");
         assert_parse_option_arg!("--foo![a|b]");
         assert_parse_option_arg!("--foo![`_foo`]");
-        assert_parse_option_arg!("--foo![=a|b]", "--foo[=a|b]");
+        assert_parse_option_arg!("--foo![=a|b]");
         assert_parse_option_arg!("--foo+[a|b]");
         assert_parse_option_arg!("--foo+[`_foo`]");
-        assert_parse_option_arg!("--foo+[=a|b]", "--foo*[=a|b]");
+        assert_parse_option_arg!("--foo+[=a|b]");
         assert_parse_option_arg!("--foo+,[a|b]");
         assert_parse_option_arg!("--foo*[a|b]");
         assert_parse_option_arg!("--foo*[=a|b]");
@@ -753,10 +780,10 @@ mod tests {
         assert_parse_option_arg!("-foo[`_foo`]");
         assert_parse_option_arg!("-foo![a|b]");
         assert_parse_option_arg!("-foo![`_foo`]");
-        assert_parse_option_arg!("-foo![=a|b]", "-foo[=a|b]");
+        assert_parse_option_arg!("-foo![=a|b]");
         assert_parse_option_arg!("-foo+[a|b]");
         assert_parse_option_arg!("-foo+[`_foo`]");
-        assert_parse_option_arg!("-foo+[=a|b]", "-foo*[=a|b]");
+        assert_parse_option_arg!("-foo+[=a|b]");
         assert_parse_option_arg!("-foo*[a|b]");
         assert_parse_option_arg!("-foo*[=a|b]");
         assert_parse_option_arg!("-foo*[`_foo`]");
@@ -782,7 +809,9 @@ mod tests {
         assert_parse_option_arg!("-f[`_foo`]");
         assert_parse_option_arg!("-f![a|b]");
         assert_parse_option_arg!("-f![`_foo`]");
-        assert_parse_option_arg!("-f![=a|b]", "-f[=a|b]");
+        assert_parse_option_arg!("-f![=a|b]");
+        assert_parse_option_arg!("-D-");
+        assert_parse_option_arg!("-D--");
     }
 
     #[test]
@@ -832,10 +861,10 @@ mod tests {
         assert_parse_positional_arg!("foo[=a|b]");
         assert_parse_positional_arg!("foo![a|b]");
         assert_parse_positional_arg!("foo![`_foo`]");
-        assert_parse_positional_arg!("foo![=a|b]", "foo[=a|b]");
+        assert_parse_positional_arg!("foo![=a|b]");
         assert_parse_positional_arg!("foo+[a|b]");
         assert_parse_positional_arg!("foo+[`_foo`]");
-        assert_parse_positional_arg!("foo+[=a|b]", "foo*[=a|b]");
+        assert_parse_positional_arg!("foo+[=a|b]");
         assert_parse_positional_arg!("foo*[a|b]");
         assert_parse_positional_arg!("foo*[`_foo`]");
         assert_parse_positional_arg!("foo*[=a|b]");
@@ -852,8 +881,8 @@ mod tests {
         assert_token!("# @version 1.0.0", Version, "1.0.0");
         assert_token!("# @author Somebody", Author, "Somebody");
         assert_token!("# @cmd A subcommand", Cmd, "A subcommand");
-        assert_token!("# @alias tst", Aliases, vec!["tst"]);
-        assert_token!("# @alias t,tst", Aliases, vec!["t", "tst"]);
+        assert_token!("# @alias tst", Aliases, ["tst"]);
+        assert_token!("# @alias t,tst", Aliases, ["t", "tst"]);
         assert_token!("# @flag -f --foo", FlagOption);
         assert_token!("# @option -f --foo", FlagOption);
         assert_token!("# @arg foo", Positional);
