@@ -18,10 +18,6 @@ pub fn to_cobol_case(value: &str) -> String {
         .convert(value)
 }
 
-pub fn hyphens_to_underscores(name: &str) -> String {
-    name.replace('-', "_")
-}
-
 pub fn escape_shell_words(value: &str) -> String {
     shell_words::quote(value).to_string()
 }
