@@ -5,13 +5,14 @@ fn multiple() {
     let script = r###"
 # @flag   -f --fc*
 # @option -o --oa* <DIR>
+# @option -b --ob* <CMD> <DIR>
 # @option -d --od <DIR> <FILE>
 # @arg var* <FILE>
 "###;
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", ""],
             vec!["prog", "-"],
             vec!["prog", "--"],
@@ -21,8 +22,11 @@ fn multiple() {
             vec!["prog", "-o", ""],
             vec!["prog", "-o", "d1"],
             vec!["prog", "-o", "d1", ""],
-            vec!["prog", "-o", "d1", "d2"],
-            vec!["prog", "-o", "d1", "d2", ""],
+            vec!["prog", "-b", ""],
+            vec!["prog", "-b", "d1"],
+            vec!["prog", "-b", "d1", ""],
+            vec!["prog", "-b", "d1", "d2"],
+            vec!["prog", "-b", "d1", "d2", ""],
             vec!["prog", "-d", "d1"],
             vec!["prog", "-d", "d1", ""],
             vec!["prog", "-d", "d1", "d2"],
@@ -48,7 +52,7 @@ fn shorts() {
 
     snapshot_compgen!(
         SCRIPT,
-        vec![
+        [
             vec!["prog", ""],
             vec!["prog", "-"],
             vec!["prog", "--"],
@@ -75,11 +79,13 @@ fn subcmds() {
 cmda() { :; }
 # @cmd
 cmdb() { :; }
+# @cmd
+cmdbc() { :; }
 "###;
 
     snapshot_compgen!(
         SCRIPT,
-        vec![
+        [
             vec!["prog", ""],
             vec!["prog", "c"],
             vec!["prog", "cmda"],
@@ -87,6 +93,7 @@ cmdb() { :; }
             vec!["prog", "help", ""],
             vec!["prog", "help", "c"],
             vec!["prog", "help", "cmda", ""],
+            vec!["prog", "help", "cmdb"],
         ]
     );
 }
@@ -105,7 +112,7 @@ cmd::subb() { :; }
 
     snapshot_compgen!(
         SCRIPT,
-        vec![
+        [
             vec!["prog", ""],
             vec!["prog", "cmd"],
             vec!["prog", "cmd", ""],
@@ -118,6 +125,41 @@ cmd::subb() { :; }
     );
 }
 
+#[test]
+fn flag_cmds() {
+    const SCRIPT: &str = r###"
+# @option -G
+
+# @cmd Run --foo
+# @alias -F
+# @flag --fa
+--foo() {
+    :;
+}
+
+# @cmd Run bar
+# @alias -B
+# @flag -C
+# @flag -D
+# @flag --fa
+bar() {
+    :;
+}
+"###;
+
+    snapshot_compgen!(
+        SCRIPT,
+        [
+            vec!["prog", ""],
+            vec!["prog", "-"],
+            vec!["prog", "-B"],
+            vec!["prog", "-B", "-"],
+            vec!["prog", "-BC"],
+            vec!["prog", "-G"],
+        ]
+    );
+}
+
 #[test]
 fn positionals() {
     let script = r###"
@@ -140,7 +182,7 @@ cmdc() { :; }
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "cmda", ""],
             vec!["prog", "cmda", "v1"],
             vec!["prog", "cmda", "v1", ""],
@@ -164,7 +206,7 @@ cmdc() { :; }
 
 #[test]
 fn choice() {
-    let script = r###"
+    let script = r#"
 # @option --oa[`_choice_fn`]
 # @option --ob[x|y|z]
 # @option --oc*,[`_choice_fn`]
@@ -173,11 +215,11 @@ fn choice() {
 _choice_fn() {
 	echo -e "abc\ndef\nghi"
 }
-"###;
+"#;
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "--oa", ""],
             vec!["prog", "--oa="],
             vec!["prog", "--oa=a"],
@@ -197,17 +239,14 @@ _choice_fn() {
 
 #[test]
 fn choice_multi() {
-    let script = r###"
+    let script = r#"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
 	echo -e "abc\ndef\nghi"
 }
-"###;
+"#;
 
-    snapshot_compgen!(
-        script,
-        vec![vec!["prog", "--oa", ""], vec!["prog", "--oa="],]
-    );
+    snapshot_compgen!(script, [vec!["prog", "--oa", ""], vec!["prog", "--oa="],]);
 }
 
 #[test]
@@ -222,7 +261,7 @@ _choice_fn() {
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "argc"],
             vec!["prog", "argc", ""],
             vec!["prog", "argc", "argc"],
@@ -243,7 +282,7 @@ _choice_fn() {
     echo $1
 }
 "###;
-    snapshot_compgen!(script, vec![vec!["prog", "cmd", "a\\b", ""],]);
+    snapshot_compgen!(script, [vec!["prog", "cmd", "a\\b", ""],]);
 }
 
 #[test]
@@ -253,7 +292,7 @@ fn option_multi_vals() {
 "###;
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "--oa", ""],
             vec!["prog", "--oa", "bash", ""],
             vec!["prog", "--oa", "bash", "cmd1", ""],
@@ -275,7 +314,7 @@ cmda() { :; }
 # @cmd line
 cmdb() { :; }
 "###;
-    snapshot_compgen!(script, vec![vec!["prog", ""], vec!["prog", "cmda", ""],]);
+    snapshot_compgen!(script, [vec!["prog", ""], vec!["prog", "cmda", ""],]);
 }
 
 #[test]
@@ -284,7 +323,7 @@ fn no_param() {
 # @cmd
 cmd() { :; }
 "###;
-    snapshot_compgen!(script, vec![vec!["prog", "cmd", ""],]);
+    snapshot_compgen!(script, [vec!["prog", "cmd", ""],]);
 }
 
 #[test]
@@ -300,7 +339,7 @@ cmdb() { :; }
 "###;
     snapshot_compgen!(
         script,
-        vec![vec!["prog", "cmda", ""], vec!["prog", "cmdb", ""],]
+        [vec!["prog", "cmda", ""], vec!["prog", "cmdb", ""],]
     );
 }
 
@@ -310,7 +349,7 @@ fn one_combine_shorts() {
 # @flag -a
 # @flag -b
 "###;
-    snapshot_compgen!(script, vec![vec!["prog", "-a"],]);
+    snapshot_compgen!(script, [vec!["prog", "-a"],]);
 }
 
 #[test]
@@ -324,7 +363,7 @@ cmdb() { :; }
 "###;
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", ""],
             vec!["prog", "cmdx", ""],
             vec!["prog", "cmdx", "cmd"]
@@ -338,7 +377,7 @@ fn just_match() {
 # @option --oa
 # @option --oa-file
 "###;
-    snapshot_compgen!(script, vec![vec!["prog", "--oa"]]);
+    snapshot_compgen!(script, [vec!["prog", "--oa"]]);
 }
 
 #[test]
@@ -358,7 +397,7 @@ arg() { :; }
 
     snapshot_compgen!(
         script,
-        vec![vec!["prog", "no_arg", ""], vec!["prog", "arg", ""]]
+        [vec!["prog", "no_arg", ""], vec!["prog", "arg", ""]]
     );
 }
 
@@ -374,7 +413,7 @@ cmda() { :; }
 cmdb() { :; }
 "###;
 
-    snapshot_compgen!(SCRIPT, vec![vec!["prog", ""],]);
+    snapshot_compgen!(SCRIPT, [vec!["prog", ""],]);
 }
 
 #[test]
@@ -387,26 +426,45 @@ fn one_subcmd_with_options() {
 cmda() { :; }
 "###;
 
-    snapshot_compgen!(SCRIPT, vec![vec!["prog", ""],]);
+    snapshot_compgen!(SCRIPT, [vec!["prog", ""],]);
+}
+
+#[test]
+fn dashes_at() {
+    let script = r#"
+# @arg val*[`_choice_fn`]
+_choice_fn() {
+    if [[ -z "$argc__dashes" ]]; then
+        echo -e "abc\ndef\nghi"
+    else
+        echo -e "v1\nv2"
+    fi
+}
+"#;
+
+    snapshot_compgen!(
+        script,
+        [vec!["prog", "abc", ""], vec!["prog", "abc", "--", ""],]
+    );
 }
 
 #[test]
 fn no_space() {
-    let script = r###"
+    let script = r#"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
 	echo -e "abc"
 	echo -e "def\0"
 	echo -e "ghk\thello world"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", ""]);
 }
 
 #[test]
 fn suffix() {
-    let script = r###"
+    let script = r#"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
     echo -e "__argc_suffix==\0"
@@ -414,14 +472,14 @@ _choice_fn() {
 	echo -e "def"
 	echo -e "ghk"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", ""]);
 }
 
 #[test]
 fn value_display() {
-    let script = r###"
+    let script = r#"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
 	echo "abc:def:xyz"
@@ -429,14 +487,14 @@ _choice_fn() {
 	echo "abc:ijk:abc"
 	echo "abc:ijk:xyz"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa="]);
+    snapshot_compgen_shells!(script, ["prog", "--oa="]);
 }
 
 #[test]
 fn multi_parts() {
-    let script = r###"
+    let script = r#"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
     echo __argc_prefix=A/
@@ -444,9 +502,9 @@ _choice_fn() {
 	echo B
 	echo -e "B/\0"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", "A/"]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", "A/"]);
 }
 
 #[test]
@@ -462,23 +520,27 @@ _choice_fn() {
 }
 "###;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", "A/B/"]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", "A/B/"]);
 }
 
 #[test]
 fn assing_option_value() {
-    let script = r###"
+    let script = r"
 # @option --oa[`_choice_fn`]
 # @arg val[`_choice_fn`]
 _choice_fn() {
     echo __argc_filter=
-    ( set -o posix ; set ) | grep 'argc_\|ARGC_COMPGEN\|ARGC_FILTER\|ARGC_LAST_ARG\|ARGC_VARS' 
+    ( set -o posix ; set ) | grep 'argc_\|ARGC_COMPGEN\|ARGC_CWORD\|ARGC_LAST_ARG\|ARGC_VARS' 
 }
-"###;
+";
 
     snapshot_compgen!(
         script,
-        vec![vec!["prog", "--oa=abc"], vec!["prog", "oa=abc"],]
+        [
+            vec!["prog", "--oa=abc"],
+            vec!["prog", "oa=abc"],
+            vec!["prog", "--", "--oa=abc"]
+        ]
     );
 }
 
@@ -496,7 +558,7 @@ _choice_fn() {
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["sudo", "cmd", ""],
             vec!["sudo", "cmd", "-"],
             vec!["sudo", "cmd", "--"],
@@ -519,7 +581,7 @@ _choice_fn() {
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "--oa"],
             vec!["prog", "--oa", ""],
             vec!["prog", "--oa", "--"],
@@ -529,6 +591,31 @@ _choice_fn() {
     );
 }
 
+#[test]
+fn option_prefixed() {
+    let script = r###"
+# @option -D-[`_choice_fn`]
+# @option -X --ox-[`_choice_fn`]
+_choice_fn() {
+    echo VAR1=value1 
+    echo VAR2=value2
+    echo VAR3
+}
+"###;
+
+    snapshot_compgen!(
+        script,
+        [
+            vec!["prog", "-D"],
+            vec!["prog", "-DVAR1"],
+            vec!["prog", "-X"],
+            vec!["prog", "-XVAR1"],
+            vec!["prog", "--ox"],
+            vec!["prog", "--ox", "VAR1"],
+        ]
+    );
+}
+
 #[test]
 fn last_arg_option_assign() {
     let script = r###"
@@ -543,7 +630,7 @@ _choice_fn() {
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "--ob="],
             vec!["prog", "--oa", "--ob="],
             vec!["prog", "abc", "--ob="],
@@ -569,7 +656,7 @@ cmd() {
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "args", ""],
             vec!["prog", "args", "v"],
             vec!["prog", "cmd", "--file", ""],
@@ -587,7 +674,7 @@ fn redirect_symbols() {
 # @arg text*
 "###;
 
-    snapshot_compgen!(script, vec![vec!["prog", ">", "Argc"]], argc::Shell::Bash);
+    snapshot_compgen!(script, [vec!["prog", ">", "Argc"]], argc::Shell::Bash);
 }
 
 #[test]
@@ -602,62 +689,62 @@ _choice_delegate() {
 
     snapshot_compgen!(
         script,
-        vec![vec!["prog", "abc"], vec!["prog", "-a"], vec!["prog", "-"]],
+        [vec!["prog", "abc"], vec!["prog", "-a"], vec!["prog", "-"]],
         argc::Shell::Bash
     );
 }
 
 #[test]
 fn mult_char() {
-    let script = r###"
+    let script = r#"
 # @option --oa*,[`_choice_fn`]
 _choice_fn() {
     echo -e "abc\ndef\nijk"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", "abc,"]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", "abc,"]);
 }
 
 #[test]
 fn mult_char2() {
-    let script = r###"
+    let script = r#"
 # @option --oa*,[`_choice_fn`]
 _choice_fn() {
     echo -e "abc\ndef\nijk"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa=abc,"]);
+    snapshot_compgen_shells!(script, ["prog", "--oa=abc,"]);
 }
 
 #[test]
 fn starts_quote() {
-    let script = r###"
+    let script = r#"
 # @option --oa[`_choice_fn`]
 _choice_fn() {
     echo -e "abc\ndef\nijk"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa='"]);
+    snapshot_compgen_shells!(script, ["prog", "--oa='"]);
 }
 
 #[test]
 fn starts_quote2() {
-    let script = r###"
+    let script = r#"
 # @option --oa[`_choice_fn`]
 _choice_fn() {
     echo -e "abc\ndef\nijk"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "'--oa="]);
+    snapshot_compgen_shells!(script, ["prog", "'--oa="]);
 }
 
 #[test]
 fn desc() {
-    let script = r###"
+    let script = r#"
 # @option --oa[`_choice_fn`]
 _choice_fn() {
     echo -e "abc\tdesc"
@@ -666,40 +753,40 @@ _choice_fn() {
     echo -e " xyz \t[desc]"
     echo -e " cjk\t福聲幸雪弓們家扒乍植哪黑信，坡也士背文反四未間美穿八和經。何朵申別兆洋行苗青誰圓弓葉福音語：向哭扒長次友誰員完"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", ""]);
 }
 
 #[test]
 fn desc2() {
-    let script = r###"
+    let script = r#"
 # @option --oa[`_choice_fn`]
 _choice_fn() {
     echo -e "abc\t(desc1)"
     echo -e "def\t(desc2)"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", "a"]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", "a"]);
 }
 
 #[test]
 fn escape() {
-    let script = r###"
+    let script = r#"
 # @option --oa[`_choice_fn`]
 _choice_fn() {
     echo -e "a:b>c"
     echo -e "d:e>f"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
+    snapshot_compgen_shells!(script, ["prog", "--oa", ""]);
 }
 
 #[test]
 fn bash_shell() {
-    let script = r###"
+    let script = r#"
 # @option --oa[`_choice_fn`]
 # @option --ob[`_choice_fn2`]
 # @option --oc <file>
@@ -716,11 +803,11 @@ _choice_fn2() {
     echo -e "B"
     echo -e "B/\0"
 }
-"###;
+"#;
 
     snapshot_compgen!(
         script,
-        vec![
+        [
             vec!["prog", "--oa", ""],
             vec!["prog", "--oa", "abc:"],
             vec!["prog", "--ob", "/A/B"],
@@ -733,7 +820,7 @@ _choice_fn2() {
 
 #[test]
 fn kinds_of() {
-    let script = r###"
+    let script = r#"
 # @arg val[`_choice_fn`]
 _choice_fn() {
     echo -e "a1\tdesc a1"
@@ -743,30 +830,30 @@ _choice_fn() {
     echo -e "e1"
     echo -e "f1\t/color:yellow\tdesc f1"
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", ""]);
+    snapshot_compgen_shells!(script, ["prog", ""]);
 }
 
 #[test]
 fn filter_quote() {
-    let script = r###"
+    let script = r#"
 # @arg args[`_choice_fn`]
 _choice_fn() {
-    echo "__argc_prefix=${ARGC_FILTER%%=*}="
-    echo "__argc_filter=${ARGC_FILTER#*=}"
+    echo "__argc_prefix=${ARGC_CWORD%%=*}="
+    echo "__argc_filter=${ARGC_CWORD#*=}"
     echo foo
     echo bar
     :;
 }
-"###;
+"#;
 
-    snapshot_compgen_shells!(script, vec!["prog", "v='"]);
+    snapshot_compgen_shells!(script, ["prog", "v='"]);
 }
 
 #[test]
 fn color() {
-    let script = r###"
+    let script = r#"
 # @arg val[`_fn_color`]
 _fn_color() {
     echo -e "kindFlag\0\t/color:cyan"
@@ -796,8 +883,8 @@ _fn_color() {
     echo -e "colorDefault\0\t/color:default"
     echo -e "colorDefaultBold\0\t/color:default,bold"
 }
-"###;
-    snapshot_compgen_shells!(script, vec!["prog", ""]);
+"#;
+    snapshot_compgen_shells!(script, ["prog", ""]);
 }
 
 mod filedir {
@@ -817,7 +904,7 @@ mod filedir {
     fn value_name() {
         snapshot_compgen!(
             VALUE_NAME_SCRIPT,
-            vec![
+            [
                 vec!["prog", "--oa", "src/"],
                 vec!["prog", "--oa", "src/p"],
                 vec!["prog", "--oa", "./src/"],
@@ -835,7 +922,7 @@ mod filedir {
     fn value_name_win() {
         snapshot_compgen!(
             VALUE_NAME_SCRIPT,
-            vec![
+            [
                 vec!["prog", "--oa", "src\\"],
                 vec!["prog", "--oa", "src\\p"],
                 vec!["prog", "--oa", ".\\src\\"],
@@ -848,7 +935,7 @@ mod filedir {
         );
     }
 
-    const CD_SCRIPT: &str = r###"
+    const CD_SCRIPT: &str = r#"
 # @option --oa[`_choice_oa`]
 # @option --ob[`_choice_ob`]
 # @arg val[`choice_val`]
@@ -870,14 +957,14 @@ choice_val() {
 
     fi
 }
-    "###;
+    "#;
 
     #[cfg(not(windows))]
     #[test]
     fn cd() {
         snapshot_compgen!(
             CD_SCRIPT,
-            vec![
+            [
                 vec!["prog", "--oa", ""],
                 vec!["prog", "--oa="],
                 vec!["prog", "--ob", ""],
@@ -892,7 +979,7 @@ choice_val() {
     fn cd_win() {
         snapshot_compgen!(
             CD_SCRIPT,
-            vec![
+            [
                 vec!["prog", "--oa", ""],
                 vec!["prog", "--ob", ""],
                 vec!["prog", "--oa="],
