@@ -20,19 +20,17 @@ fn option_help() {
 fn option_eval() {
     snapshot_multi!(
         SCRIPT_OPTIONS,
-        vec![
+        [
             vec!["prog", "cmda"],
             vec!["prog", "cmda", "-a"],
             vec!["prog", "cmda", "-f", "-f"],
             vec!["prog", "cmda", "-e", "e"],
             vec!["prog", "cmda", "--oa", "oa"],
             vec!["prog", "cmda", "--ob", "ob1", "--ob", "ob2"],
-            vec!["prog", "cmda", "--ob", "ob1", "ob2"],
-            vec!["prog", "cmda", "--oe", "ob1,ob2", "ob3"],
+            vec!["prog", "cmda", "--oe", "ob1,ob2", "--oe", "ob3"],
             vec!["prog", "cmda", "-o", "ob1", "ob2"],
             vec!["prog", "cmda", "--cc", "abc"],
             vec!["prog", "cmda", "-soa", "soa"],
-            vec!["prog", "cmda", "--ob", "a", "b", "--ob", "c"],
             vec!["prog", "cmdc"],
             vec!["prog", "cmdc", "--oe", "oe"],
             vec!["prog", "cmdc", "--of", "of"],
@@ -45,7 +43,7 @@ fn option_eval() {
 fn option_shorts() {
     snapshot_multi!(
         SCRIPT_OPTIONS,
-        vec![
+        [
             vec!["prog", "cmda", "-af"],
             vec!["prog", "cmda", "-ae", "e"],
             vec!["prog", "cmda", "-afe", "e"],
@@ -58,7 +56,7 @@ fn option_shorts() {
 fn arg_eval() {
     snapshot_multi!(
         SCRIPT_ARGS,
-        vec![
+        [
             vec!["prog", "cmdb", "v1"],
             vec!["prog", "cmdc", "v1", "v2"],
             vec!["prog", "cmdf"],
@@ -93,7 +91,7 @@ fn arg_subcmd_help() {
 fn arg_no_param() {
     snapshot_multi!(
         SCRIPT_ARGS,
-        vec![
+        [
             vec!["prog", "cmda", "v1", "v2"],
             vec!["prog", "cmda", "--o1", "-o2", "-3"]
         ]
@@ -109,7 +107,7 @@ fn arg_no_option() {
 fn arg_two_multi() {
     snapshot_multi!(
         SCRIPT_ARGS,
-        vec![
+        [
             vec!["prog", "cmdp", "a", "b", "c"],
             vec!["prog", "cmdp", "--", "a", "b", "c"],
             vec!["prog", "cmdp", "a", "--", "b", "c"],
@@ -129,7 +127,7 @@ fn dash_split() {
 "###;
     snapshot_multi!(
         script,
-        vec![
+        [
             vec!["prog", "v1", "-f", "--oa", "a"],
             vec!["prog", "v1", "-f", "--", "--oa", "a"],
         ]
@@ -176,13 +174,76 @@ fn option_to_variable() {
             "prog",
             "--flag-foo",
             "--option-foo",
-            "f1",
-            "f2",
+            "foo1",
+            "--option-foo",
+            "foo2",
             "--option-bar",
-            "foo",
+            "bar",
             "v1",
             "x1",
             "x2"
         ]
     );
 }
+
+#[test]
+fn option_terminated() {
+    let script = r###"
+# @option --oa~ <SHELL> <SCRIPT> <ARGS...> 
+# @option --ob
+"###;
+    snapshot!(script, &["prog", "--oa", "bash", "Argcfile.sh", "--ob"]);
+}
+
+#[test]
+fn option_prefixed() {
+    let script = r###"
+# @option -D-
+"###;
+    snapshot!(script, &["prog", "-D", "v1", "-Dv2=foo"]);
+}
+
+#[test]
+fn cmd_with_hyphen() {
+    let script = r###"
+# @cmd Run --foo
+# @flag --fa
+--foo() {
+    :;
+}
+
+# @cmd Run bar
+# @alias -B
+# @flag --fa
+bar() {
+    :;
+}
+"###;
+    snapshot_multi!(
+        script,
+        [vec!["prog", "--foo", "--fa"], vec!["prog", "-B", "--fa"]]
+    );
+}
+
+#[test]
+fn cmd_combine_shorts() {
+    let script = r###"
+# @cmd
+# @flag -B
+# @flag -C
+-A() {
+    :;
+}
+"###;
+    snapshot_multi!(script, [vec!["prog", "-A"], vec!["prog", "-AB"]]);
+}
+
+#[test]
+fn name_with_special_chars() {
+    let script = r###"
+# @flag --fa:foo
+# @flag --fa.bar
+# @flag --fa_baz
+"###;
+    snapshot_multi!(script, [vec!["prog", "--fa:foo", "--fa.bar", "--fa_baz"]]);
+}
